"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CircomTemplateVisitor = void 0;
const utils_1 = require("./utils");
const generated_1 = require("../generated");
const ParserError_1 = require("../errors/ParserError");
class CircomTemplateVisitor extends generated_1.CircomVisitor {
    templates;
    currentTemplate;
    constructor() {
        super();
        this.templates = {};
        this.currentTemplate = null;
    }
    visitTemplateDeclaration = (ctx) => {
        if (ctx.ID().getText() in this.templates) {
            throw new ParserError_1.ParserError({
                message: `Template name ${ctx.ID().getText()} is already in use`,
                line: ctx.start.line,
                column: ctx.start.column,
            });
        }
        this.currentTemplate = ctx.ID().getText();
        const parameters = [];
        if (ctx.args() && ctx.args().ID_list()) {
            ctx
                .args()
                .ID_list()
                .forEach((arg) => {
                parameters.push(arg.getText());
            });
        }
        this.templates[this.currentTemplate] = {
            inputs: [],
            parameters: parameters,
            isCustom: !!ctx.CUSTOM(),
        };
        ctx
            .templateBlock()
            .templateStmt_list()
            .forEach((stmt) => {
            this.visitChildren(stmt);
        });
        this.currentTemplate = null;
    };
    visitSignalDeclaration = (ctx) => {
        if (this.currentTemplate) {
            const signalDefinition = ctx.signalDefinition();
            const identifier = signalDefinition.identifier();
            const signalType = signalDefinition.SIGNAL_TYPE().getText();
            this.templates[this.currentTemplate].inputs.push({
                ...(0, utils_1.parseIdentifier)(identifier),
                type: signalType,
            });
            ctx.identifier_list().forEach((identifier) => {
                this.templates[this.currentTemplate].inputs.push({
                    ...(0, utils_1.parseIdentifier)(identifier),
                    type: signalType,
                });
            });
        }
    };
}
exports.CircomTemplateVisitor = CircomTemplateVisitor;
